@@ -182,7 +182,7 @@ struct _FsSessionPrivate
    (G_TYPE_INSTANCE_GET_PRIVATE ((o), FS_TYPE_SESSION, FsSessionPrivate))
 */
 
-G_DEFINE_ABSTRACT_TYPE(FsSession, fs_session, GST_TYPE_OBJECT);
+G_DEFINE_ABSTRACT_TYPE(FsSession, fs_session, G_TYPE_OBJECT);
 
 static void fs_session_get_property (GObject *object,
                                      guint prop_id,
@@ -270,9 +270,6 @@ fs_session_class_init (FsSessionClass *klass)
   /**
    * FsSession:codec-preferences:
    *
-   * Type: GLib.List(FsCodec)
-   * Transfer: full
-   *
    * This is the current preferences list for the local codecs. It is
    * set by the user to specify the codec options and priorities. The user may
    * change its value with fs_session_set_codec_preferences() at any time
@@ -283,6 +280,9 @@ fs_session_class_init (FsSessionClass *klass)
    * or %FS_CODEC_ID_ANY. If the encoding name is "reserve-pt", then the
    * payload type of the codec will be "reserved" and not be used by any
    * dynamically assigned payload type.
+   *
+   * Type: GLib.List(FsCodec)
+   * Transfer: full
    */
   g_object_class_install_property (gobject_class,
       PROP_CODEC_PREFERENCES,
@@ -296,9 +296,6 @@ fs_session_class_init (FsSessionClass *klass)
   /**
    * FsSession:codecs:
    *
-   * Type: GLib.List(FsCodec)
-   * Transfer: full
-   *
    * This is the list of codecs used for this session. It will include the
    * codecs and payload type used to receive media on this session. It will
    * also include any configuration parameter that must be transmitted reliably
@@ -317,6 +314,8 @@ fs_session_class_init (FsSessionClass *klass)
    * It is a #GList of #FsCodec. User must free this codec list using
    * fs_codec_list_destroy() when done.
    *
+   * Type: GLib.List(FsCodec)
+   * Transfer: full
    */
   g_object_class_install_property (gobject_class,
       PROP_CODECS,
@@ -329,9 +328,6 @@ fs_session_class_init (FsSessionClass *klass)
   /**
    * FsSession:codecs-without-config:
    *
-   * Type: GLib.List(FsCodec)
-   * Transfer: full
-   *
    * This is the same list of codecs as #FsSession:codecs without
    * the configuration information that describes the data sent. It is suitable
    * for configurations where a list of codecs is shared by many senders.
@@ -349,6 +345,8 @@ fs_session_class_init (FsSessionClass *klass)
    * It is a #GList of #FsCodec. User must free this codec list using
    * fs_codec_list_destroy() when done.
    *
+   * Type: GLib.List(FsCodec)
+   * Transfer: full
    */
   g_object_class_install_property (gobject_class,
       PROP_CODECS_WITHOUT_CONFIG,
@@ -378,7 +376,7 @@ fs_session_class_init (FsSessionClass *klass)
         G_PARAM_READABLE | G_PARAM_STATIC_STRINGS));
 
   /**
-   * FsSession:tos
+   * FsSession:tos:
    *
    * Sets the IP ToS field (and if possible the IPv6 TCLASS field
    */
@@ -600,7 +598,7 @@ fs_session_set_send_codec (FsSession *session, FsCodec *send_codec,
 /**
  * fs_session_set_codec_preferences:
  * @session: a #FsSession
- * @codec_preferences: (element-type FsCodec): a #GList of #FsCodec with the
+ * @codec_preferences: (element-type FsCodec) (allow-none): a #GList of #FsCodec with the
  *   desired configuration
  * @error: location of a #GError, or %NULL if no error occured
  *
@@ -717,9 +715,9 @@ fs_session_get_stream_transmitter_type (FsSession *session,
 /**
  * fs_session_codecs_need_resend:
  * @session: a #FsSession
- * @old_codecs: (element-type FsCodec) (transfer none):
+ * @old_codecs: (element-type FsCodec) (transfer none) (allow-none):
  *  Codecs previously retrieved from the #FsSession:codecs property
- * @new_codecs: (element-type FsCodec) (transfer none):
+ * @new_codecs: (element-type FsCodec) (transfer none) (allow-none):
  *   Codecs recently retrieved from the #FsSession:codecs property
  *
  * Some codec updates need to be reliably transmitted to the other side
